import {
	Injectable,
	InternalServerErrorException,
	NotFoundException,
} from "@nestjs/common";
import { DbService } from "apps/backend/src/app/db/db.service";
import {
	FilesForMachinesTable,
	FilesTable,
	MachinesTable,
} from "apps/backend/src/app/db/schema";
import { eq, ilike } from "drizzle-orm";

@Injectable()
export class MachinesService {
	constructor(private readonly database: DbService) {}

	/**
	 * Finds and returns a list of machines based on the given search parameters.
	 *
	 * @param {number} limit - The maximum number of machines to return.
	 * @param {number} offset - The number of machines to skip before starting to collect the result set.
	 * @param {string} searchField - The field to search within the machine names.
	 *
	 * @return {Promise<Array>} A promise that resolves to an array of found machines.
	 */
	async findMany(limit: number, offset: number, searchField: string) {
		console.log(`Searching machines. \nSearch : ${searchField}`);
		const machines = await this.database
			.use()
			.select()
			.from(MachinesTable)
			.where(ilike(MachinesTable.machineName, String(`%${searchField}%`)))
			.limit(limit)
			.offset(offset)
			.prepare("findMachineByName")
			.execute();
		console.log(`Found ${machines.length} machines.`);
		return machines;
	}

	//TODO The method to create a machine
	async create(machineName: string, machineType: string) {
		try {
			const newMachine = await this.database
				.use()
				.insert(MachinesTable)
				.values({
					machineName,
					machineType,
				})
				.prepare("createMachine")
				.execute();
			console.log("Created a new machine.", newMachine);
			return newMachine;
		} catch (e) {
			console.error(e);
			throw new InternalServerErrorException(
				"Insertion of the new machine failed, see the server console for more information.",
			);
		}
	}

	//TODO a method to delete a machine and delete the associated FilesTypeForMachine row

	/**
	 * Finds files associated with a specific machine.
	 *
	 * @param {number} limit - The maximum number of files to return.
	 * @param {number} offset - The number of files to skip before starting to return results.
	 * @param {string} searchField - The field to search within for files.
	 * @param {string} machineId - The ID of the machine to find files for.
	 * @returns {Promise<Array<Object>>} A promise that resolves to an array of files associated with the specified machine.
	 * @throws {NotFoundException} If the machine ID is not found.
	 */
	async findFilesForMachine(
		limit: number,
		offset: number,
		searchField: string,
		machineId: string,
	) {
		const existingMachine = await this.database
			.use()
			.select()
			.from(MachinesTable)
			.where(eq(MachinesTable.id, machineId))
			.prepare("findMachineById")
			.execute();
		if (existingMachine.length !== 1)
			throw new NotFoundException("Machine id not found.");

		return await this.database
			.use()
			.select({
				associatedMachine: FilesForMachinesTable.machineId,
				data: FilesTable,
			})
			.from(FilesForMachinesTable)
			.where(eq(FilesForMachinesTable.machineId, machineId))
			.limit(limit)
			.offset(offset)
			.leftJoin(FilesTable, eq(FilesTable.uuid, FilesForMachinesTable.fileId))
			.prepare("findFilesForMachineId")
			.execute();
	}
}
